package net.sergeych.crypto2

import net.sergeych.bintools.CRC

/**
 * The crc8-protected binary data manipulation. It uses crc8-bluetooth polynomial (0xA7)
 * flavor. Note that crc8 provides good protection only for relatively small data sets.
 */
object Contrail {

    /**
     *  Create a contrail by adding a crc byte to the end of [data]. The Result is always one byte longer.
     */
    fun create(data: UByteArray): UByteArray = data + CRC.crc8(data.asByteArray())

    /**
     * Check the contrail is valid.
     */
    fun isValid(data: UByteArray): Boolean = CRC.crc8(
        data.copyOfRange(0, data.size - 1).asByteArray()
    ) == data.last()


    /**
     * Check that the contrail is valid and extracts its data
     * @return the data without crc or null if it is not valid
     */
    fun unpack(contrailData: UByteArray): UByteArray? =
        if (isValid(contrailData)) contrailData.sliceArray(0..<contrailData.size - 1) else null
}