package net.sergeych.crypto2

import kotlinx.serialization.Serializable
import kotlinx.serialization.Transient

@Serializable
sealed class UniversalKey: KeyInstance {

    abstract val keyBytes: UByteArray


    @Transient
    open val magic: KeysmagicNumber = KeysmagicNumber.Unknown


    override val id by lazy { KeyId(magic, keyBytes, null) }

    // Important: id can be overridden, so we use it, not magic:
    open val label by lazy { id.keymagic.label }

    override fun equals(other: Any?): Boolean {
        return other is UniversalKey && other.keyBytes contentEquals keyBytes
    }

    override fun hashCode(): Int {
        return keyBytes.contentHashCode()
    }

    override fun toString(): String {
        val pwdSign = if (id.kdp != null) '*' else '#'
        return "🗝${label}$pwdSign$id"
    }

    companion object {
        fun newSecretKey() = SecretKey.new()
        fun newSigningKey() = SigningSecretKey.new()
        @Suppress("unused")
        fun newSymmetricKey() = SymmetricKey.new()

    }
}

open class IllegalSignatureException(text: String = "signed data is tampered or signature is corrupted") :
    IllegalStateException(text)

class ExpiredSignatureException(text: String) : IllegalSignatureException(text)