package net.sergeych.crypto2

import com.ionspin.kotlin.crypto.signature.InvalidSignatureException
import com.ionspin.kotlin.crypto.signature.Signature
import kotlinx.serialization.SerialName
import kotlinx.serialization.Serializable
import kotlinx.serialization.Transient

/**
 * Public key to verify signatures only
 */
@Serializable
@SerialName("sigb")
class VerifyingPublicKey(override val keyBytes: UByteArray) : UniversalKey(), VerifyingKey {
    /**
     * Verify the signature and return true if it is correct.
     */
    override fun verify(signature: UByteArray, message: UByteArray): Boolean = try {
        Signature.verifyDetached(signature, message, keyBytes)
        true
    } catch (_: InvalidSignatureException) {
        false
    }

    @Transient
    override val magic: KeysmagicNumber = KeysmagicNumber.defaultVerifying

    override val id by lazy { KeyId(magic, keyBytes, null, true) }
}